"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Zodios = void 0;
const axios_1 = __importDefault(require("axios"));
const api_1 = require("./plugins/api");
const utils_1 = require("./utils");
const paramsRegExp = /:([a-zA-Z_][a-zA-Z0-9_]*)/g;
/**
 * zodios api client based on axios
 */
class Zodios {
    /**
     * constructor
     * @param baseURL - the base url to use
     * @param api - the description of all the api endpoints
     * @param options - the options to setup the client API
     * @example
     *   const apiClient = new Zodios("https://jsonplaceholder.typicode.com", [
     *     {
     *       method: "get",
     *       path: "/users",
     *       description: "Get all users",
     *       parameters: [
     *         {
     *           name: "q",
     *           type: "Query",
     *           schema: z.string(),
     *         },
     *         {
     *           name: "page",
     *           type: "Query",
     *           schema: z.string().optional(),
     *         },
     *       ],
     *       response: z.array(z.object({ id: z.number(), name: z.string() })),
     *     }
     *   ]);
     */
    constructor(baseURL, api, options) {
        this.api = api;
        this.options = {
            validateResponse: true,
            usePluginApi: true,
            ...options,
        };
        if (this.options.axiosInstance) {
            this.axiosInstance = this.options.axiosInstance;
            this.axiosInstance.defaults.baseURL = baseURL;
        }
        else {
            this.axiosInstance = axios_1.default.create({
                baseURL,
            });
        }
        if (this.options.usePluginApi) {
            this.use((0, api_1.pluginApi)());
        }
    }
    get baseURL() {
        return this.axiosInstance.defaults.baseURL;
    }
    /**
     * get the underlying axios instance
     */
    get axios() {
        return this.axiosInstance;
    }
    /**
     * use a plugin to cusomize the client
     * @param plugin - the plugin to use
     */
    use(plugin) {
        plugin(this);
    }
    findEndpoint(method, path) {
        return this.api.find((e) => e.method === method && e.path === path);
    }
    validateResponse(endpoint, response) {
        return endpoint.response.parse(response);
    }
    replacePathParams(config) {
        let result = config.url;
        const params = config.params;
        if (params) {
            result = result.replace(paramsRegExp, (match, key) => key in params ? `${params[key]}` : match);
        }
        return result;
    }
    /**
     * make a request to the api
     * @param config - the config to setup zodios options and parameters
     * @returns response validated with zod schema provided in the api description
     */
    async request(config) {
        const conf = config;
        const endpoint = this.findEndpoint(config.method, config.url);
        // istanbul ignore next
        if (!endpoint) {
            throw new Error(`No endpoint found for ${config.method} ${config.url}`);
        }
        const requestConfig = {
            ...(0, utils_1.omit)(conf, ["params", "queries"]),
            url: this.replacePathParams(conf),
            params: conf.queries,
        };
        const response = await this.axiosInstance.request(requestConfig);
        if (this.options.validateResponse) {
            return this.validateResponse(endpoint, response.data);
        }
        return response.data;
    }
    /**
     * make a get request to the api
     * @param path - the path to api endpoint
     * @param config - the config to setup axios options and parameters
     * @returns response validated with zod schema provided in the api description
     */
    async get(path, config) {
        return this.request({
            ...config,
            method: "get",
            url: path,
        });
    }
    /**
     * make a post request to the api
     * @param path - the path to api endpoint
     * @param data - the data to send
     * @param config - the config to setup axios options and parameters
     * @returns response validated with zod schema provided in the api description
     */
    async post(path, data, config) {
        return this.request({
            ...config,
            method: "post",
            url: path,
            data,
        });
    }
    /**
     * make a put request to the api
     * @param path - the path to api endpoint
     * @param data - the data to send
     * @param config - the config to setup axios options and parameters
     * @returns response validated with zod schema provided in the api description
     */
    async put(path, data, config) {
        return this.request({
            ...config,
            method: "put",
            url: path,
            data,
        });
    }
    /**
     * make a patch request to the api
     * @param path - the path to api endpoint
     * @param data - the data to send
     * @param config - the config to setup axios options and parameters
     * @returns response validated with zod schema provided in the api description
     */
    async patch(path, data, config) {
        return this.request({
            ...config,
            method: "patch",
            url: path,
            data,
        });
    }
    /**
     * make a delete request to the api
     * @param path - the path to api endpoint
     * @param config - the config to setup axios options and parameters
     * @returns response validated with zod schema provided in the api description
     */
    async delete(path, data, config) {
        return this.request({
            ...config,
            method: "delete",
            url: path,
            data,
        });
    }
}
exports.Zodios = Zodios;
